@@ -5,12 +5,11 @@ use v5.14;
 use warnings;
 no warnings;
 
-use subs qw();
-use vars qw($VERSION);
+use parent qw(Hash::AsObject);
 
 use Carp;
 
-$VERSION = '0.10_01';
+our $VERSION = '0.112';
 
 =encoding utf8
 
@@ -25,13 +24,13 @@ ReturnValue - A structured return value for failure or success
 
 	sub do_something {
 		...;
-		
+
 		return ReturnValue->error(
 			value       => $your_usual_error_value,
 			description => 'Some longer description',
 			tag         => 'short_value'
 			) if $failed;
-			
+
 		return ReturnValue->success(
 			value       => $your_usual_return_value,
 			description => 'Some longer description',
@@ -47,9 +46,9 @@ ReturnValue - A structured return value for failure or success
 
 	my $result = do_something_else();
 	for( $result->tag ) {
-		when( 'tag1' ) { ... }	
-		when( 'tag2' ) { ... }	
-	
+		when( 'tag1' ) { ... }
+		when( 'tag2' ) { ... }
+
 		}
 
 =head1 DESCRIPTION
@@ -68,27 +67,28 @@ helpful with multiple ways to describe an error.
 =cut
 
 sub _new {
-	state $allowed = {
+	my $allowed = {
 		value       => 'required',
 		description => 0,
 		tag         => 0,
 		};
 
 	my( $class, %hash ) = @_;
-	foreach my $key ( keys %hash ) {
-		next if exists $allowed->{$key};
-		delete $allowed->{$key};
-		}
 
+	delete $allowed->{$_} for keys %hash;
+
+	# these are the keys that are left over after the
+	# last foreach. These are a problem if they are
+	# requires
 	foreach my $key ( keys %$allowed ) {
 		next unless $allowed->{$key};
-		next if defined $hash{$key};
-		carp ""
+		carp "required key [$key] is missing";
+		return;
 		}
-	
+
 	bless \%hash, $class;
 	}
-	
+
 =item success
 
 Create a success object
@@ -100,11 +100,13 @@ Create an error object
 =cut
 
 sub success {
-	$_[0]->success_type->_new( @_ );
+	my( $self ) = shift;
+	$self->success_type->_new( @_ );
 	}
-	
+
 sub error {
-	$_[0]->error_type->_new( @_ );
+	my( $self ) = shift;
+	$self->error_type->_new( @_ );
 	}
 
 =item value
@@ -115,7 +117,7 @@ to figure out how you want to do that.
 
 =item description
 
-A long description of the return values, 
+A long description of the return values,
 
 =item tag
 
@@ -153,11 +155,11 @@ Returns true is the result represents a success
 
 package ReturnValue::Success {
 	use parent qw(ReturnValue);
-	
+
 	sub is_error   { 0 }
 	sub is_success { 1 }
 	}
-	
+
 package ReturnValue::Error {
 	use parent qw(ReturnValue);
 
@@ -181,13 +183,13 @@ This source is in Github:
 
 =head1 AUTHOR
 
-brian d foy, C<< <bdfoy@cpan.org> >>
+brian d foy, <bdfoy@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2013, brian d foy, All Rights Reserved.
+Copyright © 2013-2018, brian d foy <bdfoy@cpan.org>. All rights reserved.
 
-You may redistribute this under the same terms as Perl itself.
+You may redistribute this under the terms of the Artistic License 2.0.
 
 =cut
 
